//
// Name: FTROBOII.PRG
// Project: Fischer Technik interfacing
// Description: ft interface driver for 30402(II) and the RoboPro
// Language: xBase++
// Libs : based on umFish40.dll and serial com lib xbTools III
//        use the newer version of umFish40.dll from umFish41.zip!!
// Author: Rene Nooteboom
// Email: r.nooteboom2@chello.nl
// Date created: 10/05/2006 10:29
// Date updated: 02/09/2006 23:04
// Remark: 1) PollingInterval is always 10ms !
//         2) only for the II-interface : nAnalogScan is every nAnalogCycles
//            default every 5 cycles the analog-channel is scanned
//
// Function FtRAD(iHandle,nAnalogInput) // ftiAX,AY,A1,A2,AV,AZ,AXS1,AXS2,AXS3
// Function FtRClearCounters(iHandle)
// Function FtRClMesIn(iHandle)
// Function FtRClMesOut(iHandle)
// Function FtRDeviceFirmwareNr(iHandle)
// Function FtRDeviceInfo(iHandle)
// Function FtRDeviceSerialNr(iHandle)
// Function FtRDeviceType(iHandle)
// Function FtRGetCounter(iHandle,nE1toE16)
// Function FtRGetIRKey(iHandle,nCode,nKeyNr) 
// Function FtRGetModeStatus(iHandle,nMotorNr)
// Function FtRGetMotors(iHandle)
// Function FtRGetVoltage(iHandle,nVoltNr) 
// Function FtRInit(nIFType,nComOrUSBNr,lAskValues,nAnalogCycles)  // all optional
// Function FtRInput(iHandle)
// Function FtRlsMessage(iHandle)
// Function FtRMotor(iHandle,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
// Function FtRMotorLeft(iHandle,nMotorNr,nSpeed)
// Function FtRMotorOff(iHandle,nMotorNr)
// Function FtRMotorOn(iHandle,nMotorNr,nSpeed)
// Function FtRMotorRight(iHandle,nMotorNr,nSpeed)
// Function FtRMotorSpeed(iHandle,nMotorNr,nDirection,nSpeed) // MON,MCW,MCCW,MOFF
// Function FtROnNr(iHandle,nO1toO16,nPower)
// Function FtROffNr(iHandle,nO1toO16)
// Function FtROutput(iHandle,nOutput)
// Function FtRRobMotor(iHandle,nMotorNr,nDirection,nSpeed,nCount)
// Function FtRRobMotors(iHandle,nMotorStatus,nSpeedStatus, ;
// Function FtRSensor(iHandle,nE1toE16)
// Function FtRSetCounter(iHandle,nE1toE16,nValue)
// Function FtRSetLamp(iHandle,nLampNr,lOnOff)
// Function FtRSetLampEx(iHandle,nLampNr,lOnOff,nPower)
// Function FtRSetModeStatus(iHandle,nMotorNr,nMode) 
// Function FtRSetMotors(iHandle,nMotorStatus) 
// Function FtRSetMotorsEx(iHandle,nMotorStatus,nSpeedSt,nSpeedSt16) 
// PROCEDURE FtRButton(iHandle,nE1toE16,iSleepIdleTime) // wait for high
// PROCEDURE FtRButtonPushed(iHandle,nE1toE16,iSleepIdleTime) // wait for low
// PROCEDURE FtRClose(iHandle)
// Procedure FtRCloseAll()
// PROCEDURE FtRMotorsOff(iHandle)
//
#include "common.ch"
#include "inkey.ch"
#include "dll.ch"
#include "xbp.ch"
#include "xbtcom.ch"
#include "noot.ch"
#include "ft.ch"
#include "umfish40.ch"
#include "ftroboii.ch"
//
Static nUmFish40Handle:=0,nUsedComPorts_:={},nUsedHandles_:={}, ;
 nUsedIFTypes_:={},cUsedPorts_:={}
//
Function FtRInit(nIFType,nComOrUSBNr,lAskValues,nAnalogCycles)  // all optional
//
// poll interval is always 10ms
// returns : iHandle and -1 if error
//
 LOCAL GetList:={},oCrt,oParent,oOwner,cComPorts_:={}, ;
  cComPort:="COMxx",i,cmsg,iHandle,nIFType_Index:=1,lUSB:=TRUE, ;
  lExit:=FALSE,nComPort:=1,nUSBSerialNr:=0
 Default nComOrUSBNr TO 0,lAskValues TO TRUE
 Default nIFType to ftiROBO_USB_Auto,nAnalogCycles to 5
 If UmFish40Handle() == 0  // umfish40.dll not yet loaded ?
  If !LoadUmFish40()
   Return -1
  EndIf
 EndIf
 nUmFish40Handle := UmFish40Handle()
 If lAskValues
  If AppType() == APPTYPE_VIO
   vstk_push()
  EndIf
  oParent := AppDesktop()
  oOwner := SetAppWindow()
  oCrt := XbpCrt():new(oParent,oOwner,{50,50},8,40,"Select type of FT-interfacing")
  oCrt:minmax := FALSE
  oCrt:sysmenu := FALSE
  oCrt:closeable := FALSE
  oCrt:fontname := CRT_FONT_NAME
  oCrt:create()
  SetAppWindow(oCrt)
  oCrt:setModalState(XBP_DISP_APPMODAL)
  SetAppFocus(oCrt)
  CLS
  Set Cursor on
  @ 1,2 to 7,35 DOUBLE
  nIFType_Index := AChoice(2,3,6,34,ftiTYPE_TEXT_)
  lExit := (LastKey() == K_ESC) 
  oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
  oCrt:destroy()
  SetAppWindow(oOwner)
  SetAppFocus(oOwner)
  If AppType() == APPTYPE_VIO
   vstk_pop()
  EndIf
  If lExit .OR. (nIFType_Index < 1)
   Return -1
  Else
   nIFType := ftiTYPE_CODES_[nIFType_Index]
  EndIf
 EndIf
 If (nIFType_Index := AScan(ftiTYPE_CODES_,nIFType)) < 1
  ALARMTONE
  MsgBox("FtRInit(): Error : Illegal Interface Type")
  Return -1
 EndIf
 lUSB := ftiTYPE_USB_[nIFType_Index]
 If !lUSB  // via com-port
  For i:=1 to MAX_COM_PORT_NR  // scan for possible COM-ports
   If (AScan(nUsedComPorts_,i)) < 1 // port not already in use ?
    If com_open(i)  // existing port ?
     com_close(i)  
     AAdd(cComPorts_,"COM"+str_zero(i,2,0))
    EndIf
   EndIf
  Next i
  If lAskValues
   cmsg := ftiTYPE_TEXT_[nIFType_Index]
   If AppType() == APPTYPE_VIO
    vstk_push()
   EndIf
   oParent := AppDesktop()
   oOwner := SetAppWindow()
   oCrt := XbpCrt():new(oParent,oOwner,{50,50},8,60,cmsg+" Configuration")
   oCrt:minmax := FALSE
   oCrt:sysmenu := FALSE
   oCrt:closeable := FALSE
   oCrt:fontname := CRT_FONT_NAME
   oCrt:create()
   SetAppWindow(oCrt)
   oCrt:setModalState(XBP_DISP_APPMODAL)
   SetAppFocus(oCrt)
   CLS
   Set Cursor on
   @ 3,2 SAY "ComPort:"
   @ 4,2 SAY "Analog Cycles(default=5):"
   @ 3,40 Get cComPort VALID {|oGet|GetListBox(oGet,cComPorts_)} Color "W/B,GR+/B"
   @ 4,40 GET nAnalogCycles PICTURE "9999999"
   Read
   lExit := (LastKey() == K_ESC)
   nComPort := Val(Right(AllTrim(cComPort),2))
   oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
   oCrt:destroy()
   SetAppWindow(oOwner)
   SetAppFocus(oOwner)
   If AppType() == APPTYPE_VIO
    vstk_pop()
   EndIf
   If lExit .OR. !IS_BETW(nComPort,1,MAX_COM_PORT_NR)
    Return -1
   Else
    nComOrUSBNr := nComPort
   EndIf
  EndIf
  If !IS_BETW(nComOrUSBNr,1,MAX_COM_PORT_NR)
   ALARMTONE
   cmsg := "FtRInit():Error:COM-portnr not between 1 and " + Str(MAX_COM_PORT_NR,2,0)
   MsgBox(cmsg)
   Return -1
  EndIf
  cComPort := "COM" + str_zero(nComOrUSBNr,2,0)
  If !IS_ELEMENT(cComPorts_,cComPort)
   cmsg := "FtRInit():Warning: " + cComPort + " not available?"
   ALARMTONE
   MsgBox(cmsg)
  EndIf
 Else // via USB
  If lAskValues
   cmsg := ftiTYPE_TEXT_[nIFType_Index]
   If AppType() == APPTYPE_VIO
    vstk_push()
   EndIf
   oParent := AppDesktop()
   oOwner := SetAppWindow()
   oCrt := XbpCrt():new(oParent,oOwner,{50,50},8,60,cmsg+" Configuration")
   oCrt:minmax := FALSE
   oCrt:sysmenu := FALSE
   oCrt:closeable := FALSE
   oCrt:fontname := CRT_FONT_NAME
   oCrt:create()
   SetAppWindow(oCrt)
   oCrt:setModalState(XBP_DISP_APPMODAL)
   SetAppFocus(oCrt)
   CLS
   Set Cursor on
   @ 3,2 SAY "USB Serial Nr (0=Auto):"
   @ 3,40 GET nUSBSerialNr PICTURE "999999999999" 
   Read
   lExit := (LastKey() == K_ESC)
   oCrt:setModalState(XBP_DISP_MODELESS) // clean up 
   oCrt:destroy()
   SetAppWindow(oOwner)
   SetAppFocus(oOwner)
   If AppType() == APPTYPE_VIO
    vstk_pop()
   EndIf
   If lExit
    Return -1
   Else
    nComOrUSBNr := nUSBSerialNr
   EndIf
  EndIf	
 EndIf
 If lUSB
  If IS_ELEMENT(ftiTYPE_RF_,nIFType) // via a RF Datalink interface
   iHandle := DllCall(nUmFish40Handle,DLL_STDCALL,"rbOpenInterfaceRF", ;
    nComOrUSBNr) 
  Else // via USB
   iHandle := DllCall(nUmFish40Handle,DLL_STDCALL,"rbOpenInterfaceUSB", ;
    nIFType,nComOrUSBNr)
  EndIf
 Else // via a COM-port
  iHandle := DllCall(nUmFish40Handle,DLL_STDCALL,"rbOpenInterfaceCOM", ;
   nIFType,nComOrUSBNr,nAnalogCycles)  
 EndIf
 If iHandle == ftiError
  ALARMTONE
  MsgBox("FtRInit(): Error : rbOpenInterface()")
  Return -1
 EndIf 
 If !lUSB
  AAdd(nUsedComPorts_,nComOrUSBNr)
 EndIf
 AAdd(nUsedHandles_,iHandle)
 AAdd(nUsedIFTypes_,nIFType)
 AAdd(cUsedPorts_,If(lUSB,"USB","COM"+str_zero(nComOrUSBNr,2,0)))
 FtROutput(iHandle,0) // everything off
 Return iHandle // -1 if error
//
PROCEDURE FtRClose(iHandle)
 LOCAL iIndex,cComPort,i2Index,nComPort
 FtRMotorsOff(iHandle)
 DllCall(nUmFish40Handle,DLL_STDCALL,"rbCloseInterface",iHandle)
 iIndex := AScan(nUsedHandles_,iHandle)
 If iIndex > 0 // clean up
  ADel(nUsedHandles_,iIndex)
  ASHRINK(nUsedHandles_)
  ADel(nUsedIFTypes_,iIndex)
  ASHRINK(nUsedIFTypes_)
  If Left((cComPort := cUsedPorts_[iIndex]),3) == "COM"
   If (nComPort := Val(Right(cComPort,2))) > 0
    If (i2Index := AScan(nUsedComPorts_,nComPort)) > 0
     ADel(nUsedComPorts_,i2Index)
     ASHRINK(nUsedComPorts_)
    EndIf
   EndIf
  EndIf
  ADel(cUsedPorts_,iIndex)
  ASHRINK(cUsedPorts_)
 EndIf
 Return
// 
Procedure FtRCloseAll()
 LOCAL i
 For i:=1 to Len(nUsedHandles_)
  FtRClose(nUsedHandles_[i])
 Next i
 Return
//
Function FtRInput(iHandle)
 LOCAL nInputStatus:=0
 nInputStatus := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetInputs", ;
  iHandle) 
 Return nInputStatus
//
Function FtROutput(iHandle,nOutput)
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetMotors",iHandle,nOutput)
 Return nErrorCode
//
Function FtRAD(iHandle,nAnalogInput) // ftiAX,AY,A1,A2,AV,AXS1,AXS2,AXS3
 LOCAL nAnalogValue:=0
 nAnalogValue := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetAnalog", ;
  iHandle,nAnalogInput)
 Return nAnalogValue
//
Function FtRGetVoltage(iHandle,nVoltNr) 
 LOCAL nVoltage:=0
 nVoltage := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetVoltage", ;
  iHandle,nVoltNr)
 Return nVoltage
//
Function FtRGetIRKey(iHandle,nCode,nKeyNr) 
 LOCAL nValue:=0
 nValue := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetIRKey", ;
  iHandle,nCode,nKeyNr)
 Return nValue
//
Function FtRSensor(iHandle,nE1toE16)
 LOCAL lSensor
 lSensor := (DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetInput", ;
  iHandle,nE1toE16) == 1)
 Return lSensor
//
PROCEDURE FtRMotorsOff(iHandle)
 FtROutput(iHandle,0)
RETURN
//
Function FtRSetCounter(iHandle,nE1toE16,nValue)
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetCounter",iHandle,nE1toE16, ;
  nValue)
Return nErrorCode
//
Function FtRClearCounters(iHandle)
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbClearCounters",iHandle)
Return nErrorCode
//
Function FtRGetCounter(iHandle,nE1toE16)
 LOCAL nCount:=0
 nCount := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetCounter",iHandle, ;
  nE1toE16)
Return nCount
//
PROCEDURE FtRButton(iHandle,nE1toE16,iSleepIdleTime)
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE !FtRSensor(iHandle,nE1toE16)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE !FtRSensor(iHandle,nE1toE16)
  EndDo	
 EndIf
RETURN
//
PROCEDURE FtRButtonPushed(iHandle,nE1toE16,iSleepIdleTime)
 Default iSleepIdleTime to 1 // 1/100s
 If iSleepIdleTime > 0
  DO WHILE FtRSensor(iHandle,nE1toE16)
   Sleep(iSleepIdleTime)
  EndDo
 Else
  DO WHILE FtRSensor(iHandle,nE1toE16)
  EndDo	
 EndIf
Return
//
Function FtRMotorSpeed(iHandle,nMotorNr,nDirection,nSpeed) // MON,MCW,MCCW,MOFF
 LOCAL nErrorCode:=0
 Default nSpeed to FULL_MOTOR_SPEED
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetMotorEx", ;
  iHandle,nMotorNr,nDirection,nSpeed)
 Return nErrorCode
//
Function FtRMotor(iHandle,nMotorNr,nDirection) // MON,MCW,MCCW,MOFF
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetMotor", ;
  iHandle,nMotorNr,nDirection)
 Return nErrorCode
//
Function FtRGetMotors(iHandle)
 LOCAL nOutputStatus:=0
 nOutputStatus := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetMotors", ;
  iHandle) 
 Return nOutputStatus
//
Function FtRGetModeStatus(iHandle,nMotorNr)
 LOCAL nMode:=0
 nMode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetModeStatus", ;
  iHandle,nMotorNr) 
 Return nMode
//
Function FtRSetModeStatus(iHandle,nMotorNr,nMode) 
  LOCAL nErrorCode
  nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetModeStatus", ;
  iHandle,nMotorNr,nMode)
 Return nErrorCode
//
Function FtRSetMotors(iHandle,nMotorStatus) 
 Return FtROutput(iHandle,nMotorStatus)
//
Function FtRSetMotorsEx(iHandle,nMotorStatus,nSpeedSt,nSpeedSt16) 
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetMotorsEx", ;
  iHandle,nMotorStatus,nSpeedSt,nSpeedSt16)
 Return nErrorCode
//
Function FtRMotorLeft(iHandle,nMotorNr,nSpeed)
 LOCAL nErrorCode:=0
 nErrorCode := FtRMotorSpeed(iHandle,nMotorNr,ftiLeft,nSpeed)
Return nErrorCode
//
Function FtRMotorRight(iHandle,nMotorNr,nSpeed)
 LOCAL nErrorCode:=0
 nErrorCode := FtRMotorSpeed(iHandle,nMotorNr,ftiRight,nSpeed)
Return nErrorCode
//
Function FtRMotorOff(iHandle,nMotorNr)
 LOCAL nErrorCode:=0
 nErrorCode := FtRMotor(iHandle,nMotorNr,ftiOff)
Return nErrorCode
//
Function FtRMotorOn(iHandle,nMotorNr,nSpeed)
 LOCAL nErrorCode:=0
 nErrorCode := FtRMotorSpeed(iHandle,nMotorNr,ftiOn,nSpeed)
Return nErrorCode
//
Function FtRSetLamp(iHandle,nLampNr,lOnOff)
 LOCAL nErrorCode:=0,nOnOff
 Default lOnOff to TRUE
 nOnOff := If(lOnOff,1,0)
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetLamp", ; 
   iHandle,nLampNr,nOnOff)
 Return nErrorCode
//
Function FtRSetLampEx(iHandle,nLampNr,lOnOff,nPower)
 LOCAL nErrorCode:=0,nOnOff
 Default lOnOff to TRUE,nPower to FULL_OUTPUT_POWER
 nOnOff := If(lOnOff,1,0)
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbSetLampEx", ; 
   iHandle,nLampNr,nOnOff,nPower)
 Return nErrorCode
//
Function FtROnNr(iHandle,nO1toO16,nPower)
 Return FtRSetLampEx(iHandle,nO1toO16,TRUE,nPower)
//
Function FtROffNr(iHandle,nO1toO16)
 Return FtRSetLamp(iHandle,nO1toO16,FALSE)
//
// RobMode
// 
//  motor    end input(Ex)   impuls input counter
//    1
//    2
//    3            5                 6
//    4            7                 8
//    5            9                10
//    6           11                12
//    7           13                14
//    8           15                16
//
// note assumes ftiLinks(0x01) directed to end input !
//
// example : rbRobMotors(iHandle,0x9,0x74,0x0,0x05)
//  motorstatus = 1001 = 0x9
//  speedstatus = 0111 0100 = 0x7
//  modestatus  = 0100 = 0x5
//
//  -> M2 right(=10),speed 7(=0111),rob mode(=01)
//     M1 left(=01),speed 4(=0100),normal mode(=00)
//
Function FtRRobMotors(iHandle,nMotorStatus,nSpeedStatus, ;
  nSpeedStatus16,nModeStatus)
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbRobMotors", ;
  iHandle,nMotorStatus,nSpeedStatus,nSpeedStatus16,nModeStatus)
 Return nErrorCode
//
Function FtRRobMotor(iHandle,nMotorNr,nDirection,nSpeed,nCount)
 LOCAL nErrorCode:=0
 nErrorCode := DllCall(nUmFish40Handle,DLL_STDCALL,"rbRobMotor", ;
  iHandle,nMotorNr,nDirection,nSpeed,nCount)
 Return nErrorCode
//
Function FtRDeviceType(iHandle)
 LOCAL nType:=0
 nType := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetActDeviceType", ;
  iHandle) 
 Return nType
//
Function FtRDeviceSerialNr(iHandle)
 LOCAL nSerialNr:=0
 nSerialNr := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetActDeviceSerialNr", ;
  iHandle) 
 Return nSerialNr
//
Function FtRDeviceFirmwareNr(iHandle)
 LOCAL nFirmwareNr:=0
 nFirmwareNr := DllCall(nUmFish40Handle,DLL_STDCALL,"rbGetActDeviceFirmwareNr", ;
  iHandle) 
 Return nFirmwareNr
//
Function FtRDeviceInfo(iHandle)
 LOCAL iIndex,nIFType,i2Index,cInfo:=""
 If (iIndex := AScan(nUsedHandles_,iHandle)) > 0
  nIFType := nUsedIFTypes_[iIndex]
  cInfo := "[" + cUsedPorts_[iIndex] + "]"
  If (i2Index := AScan(ftiTYPE_CODES_,nIFType)) > 0
   cInfo += ftiTYPE_TEXT_[i2Index] 
  EndIf
 EndIf
 Return cInfo
//
Function FtRClMesIn(iHandle)
 LOCAL nErrorNr:=0
 nErrorNr := DllCall(nUmFish40Handle,DLL_STDCALL,"rbClearMessagesIn", ;
  iHandle) 
 Return nErrorNr
//
Function FtRClMesOut(iHandle)
 LOCAL nErrorNr:=0
 nErrorNr := DllCall(nUmFish40Handle,DLL_STDCALL,"rbClearMessagesOut", ;
  iHandle) 
 Return nErrorNr
//
Function FtRlsMessage(iHandle)
 LOCAL nCount:=0
 nCount := DllCall(nUmFish40Handle,DLL_STDCALL,"rblsMessage",iHandle) 
 Return nCount
//
// einde Ftroboii.prg
//
