//
//   Program: NOOT.prg
//  language: xBase++
//    Author: Rene Nooteboom
//     Email: r.nooteboom2@chello.nl
// Copyright: Copyright 2006
//      Date: 11-3-2006
//   updated: 19/08/2006 00:13
//      Type: generic noot library
//     needs: simcato.prg or xbtoolsIII library
//

//
// FUNCTION abinsrch(marray,search_for) RETURN nindex
// FUNCTION acolumn(aarray,ncolumn,nstart,nstop) RETURN column_
// FUNCTION afind(array_,bblock) RETURN xelement
// FUNCTION ainitfill(aarray_,xvalue) RETURN NIL
// FUNCTION amove(aarray,nfrom,ndest,ptarget) RETURN lResult
// FUNCTION aparse(cstring,cdelimiter) RETURN clist_
// FUNCTION arest_array(cfilename) RETURN array_
// FUNCTION asave_array(aarray_,cfilename) RETURN lsuccess
// FUNCTION asum(aarray,nstart,nstop) RETURN nsum
// Function atos(avalues,apictures,cseparator) RETURN cstring
// FUNCTION BinaryString(nByte) // 8765 4321 RETURN cByte
// FUNCTION BitwiseAND(cByte1,cByte2) RETURN cByte
// FUNCTION BitwiseOR(cByte1,cByte2) RETURN cByte
// FUNCTION ClearBitOfByte(nByte,nBitNr) // 8765 4321 RETURN nOutputByte
// FUNCTION clearinival() RETURN NIL
// FUNCTION cspnout(cstring,callowed) RETURN npos
// FUNCTION decrypt(cstring,cencr_key) RETURN cdecrypted
// FUNCTION deleteinival(cidentifier) RETURN NIL
// FUNCTION ElapsedTime() RETURN nElapsedTime
// Function encrypt(cstring,cencr_key) RETURN cencrypted
// FUNCTION encryptkey(cnewkey) RETURN coldkey
// Function GetListBox(oGet,clist_)RETURN lvalid
// Procedure HiddenReader(oGet)
// FUNCTION ini_exist(cidentifier) RETURN lexist
// FUNCTION ini_value(cidentifier,xdef_value) RETURN xvalue
// Function isall(xvalue,xchar) RETURN lall
// FUNCTION IsBitSet(nByte,nBitNr) // 8765 4321 RETURN lSet
// FUNCTION makeinival(cidentifier,xvalue) RETURN NIL
// FUNCTION makenumeric(xvalue,linteger) RETURN nvalue
// FUNCTION NumByteAnd(nByte1,nByte2) RETURN nbyte
// FUNCTION NumByteOr(nByte1,nByte2) RETURN nbyte
// FUNCTION NumericValue(cByte) // 8765 4321 RETURN ndecimaal
// FUNCTION readinival(cfilename) RETURN NIL
// FUNCTION SetBitOfByte(nByte,nBitNr) // 8765 4321 RETURN nOutputByte
// FUNCTION spl_into_lines(string,line_len,max_lines) RETURN clines_
// Procedure StartStopWatch()
// FUNCTION str_zero(nnumber,nwidth,ndecimals) RETURN czeropadded
// FUNCTION stringify(xobject) RETURN cstring
// FUNCTION vstk_pop() RETURN NIL
// FUNCTION vstk_push() RETURN NIL
// FUNCTION win_restore(cwin_str) RETURN NIL
// FUNCTION win_save(t,l,b,r) RETURN cwindowstring
// FUNCTION writeinival(cfilename) RETURN lok
// FUNCTION zeropad(xvalue,nlength,ckar,cposition) RETURN cpaddedstring
//

#include "common.ch"
#include "inkey.ch"
#include "fileio.ch"
#include "noot.ch"

#define I_ROW     1
#define I_COLUMN  2
#define I_CURSOR  3
#define I_COLOR   4

#define cZERO "0"
#define cBINAIR "01"

#define INISEPARATOR "="

#define ENCRYPT_KEY "NOOTKEUL"

STATIC video_stack_ := {} // stack for vstk_pop(),vstk_push()

Static asini := {}  // array for .ini system functions

STATIC ccryptkey:=ENCRYPT_KEY // see encrypt(),decrypt()

Static nStartTime:=0

//
// VOID vstk_push()
//
// push an entry on the video stack save the state of SET CURSOR
// the current cursor row/column and the current color
//

FUNCTION vstk_push()
 aadd(video_stack_,{row(),col(),setcursor(),setcolor()})
RETURN NIL

//
// VOID vstk_pop()
//
// remove entry from videostack popping it
// and restore video-attributes
//

FUNCTION vstk_pop()
 LOCAL len_stack := len(video_stack_)
 IF len_stack > 0
  setpos(video_stack_[len_stack][I_ROW],video_stack_[len_stack][I_COLUMN])
  setcursor(video_stack_[len_stack][I_CURSOR])
  setcolor(video_stack_[len_stack][I_COLOR])
  asize(video_stack_,len(video_stack_)-1)
 ENDIF
RETURN NIL

//
// CHARACTER win_save(t,l,b,r)
//
// NUMERIC t,l,b,r // window coordinates default whole screen
//
// RETURN coordinates and screenarea as a string
//
//
// save a window area in a string
//

FUNCTION win_save(t,l,b,r)
 DEFAULT t TO 0,l TO 0
 DEFAULT b TO maxrow(),r TO maxcol()
RETURN (chr(t)+chr(l)+chr(b)+chr(r)+savescreen(t,l,b,r))

//
// VOID win_restore(win_str)
//
// CHARACTER win_str ,window string created by win_save()
//
// restore a window area with win_save()
//

FUNCTION win_restore(cwin_str)
 LOCAL t,l,b,r
 t := asc(substr(cwin_str,1,1))
 l := asc(substr(cwin_str,2,1))
 b := asc(substr(cwin_str,3,1))
 r := asc(substr(cwin_str,4,1))
 restscreen(t,l,b,r,substr(cwin_str,5))
RETURN NIL

//
// collection of bit/byte tool functions
//

FUNCTION BinaryString(nByte) // 8765 4321
 LOCAL i,nValue,cByte:=""
 FOR i := 8 TO 1 STEP -1
  nValue := int(nByte/(2 ^ (i-1)))
  cByte := cByte + substr(cBINAIR,nValue+1,1)
  nByte -= nValue * ( 2 ^ (i-1))
  IF nByte <= 0
   cByte := padr(cByte,8,cZERO)
   EXIT 
  ENDIF
 NEXT i
 RETURN cByte
 
FUNCTION NumericValue(cByte) // 8765 4321
 LOCAL i,nDecimaal:=0
 cByte := padl(alltrim(cByte),8,cZERO)
 FOR i := 8 TO 1 STEP -1
  IF substr(cByte,9-i,1) == "1"
   nDecimaal += 2 ^ ( i - 1 )
  ENDIF
 NEXT i
 RETURN nDecimaal
 
FUNCTION BitwiseAND(cByte1,cByte2)
 LOCAL i,cByte:=""
 FOR i:=1 TO 8
  IF substr(cByte1,i,1) == "1" .AND. substr(cByte2,i,1) == "1"
   cByte := cByte + "1"
  ELSE
   cByte := cByte + cZERO      
  ENDIF
 NEXT i
 RETURN cByte

FUNCTION BitwiseOR(cByte1,cByte2)
 LOCAL i,cByte:=""
 FOR i:=1 TO 8
  IF substr(cByte1,i,1) == cZERO .AND. substr(cByte2,i,1) == cZERO
   cByte := cByte + cZERO
  ELSE
   cByte := cByte + "1"      
  ENDIF
 NEXT i
 RETURN cByte
 
FUNCTION NumByteAnd(nByte1,nByte2)
 LOCAL nByte:=0,cByte1,cByte2,cByte
 cByte1 := BinaryString(nByte1)
 cByte2 := BinaryString(nByte2)
 cByte := BitwiseAND(cByte1,cByte2)
 nByte := NumericValue(cByte)
RETURN nByte
 
FUNCTION NumByteOr(nByte1,nByte2)
 LOCAL nByte:=0,cByte1,cByte2,cByte
 cByte1 := BinaryString(nByte1)
 cByte2 := BinaryString(nByte2)
 cByte := BitwiseOR(cByte1,cByte2)
 nByte := NumericValue(cByte)
RETURN nByte 

FUNCTION IsBitSet(nByte,nBitNr) // 8765 4321
 LOCAL lSet:=FALSE,cByte
 cByte := BinaryString(nByte)
 lSet := (substr(cByte,9-nBitNr,1) == "1")
RETURN lSet

FUNCTION ClearBitOfByte(nByte,nBitNr) // 8765 4321
 LOCAL i,cByte,cOutputByte:="",nOutputByte
 cByte := BinaryString(nByte)
 FOR i := 8 TO 1 STEP -1
  IF i == nBitNr
   cOutputByte := cOutputByte + cZERO
  ELSE
   cOutputByte := cOutputByte + substr(cByte,9-i,1)
  ENDIF  
 NEXT i
 nOutputByte := NumericValue(cOutputByte)
RETURN nOutputByte

FUNCTION SetBitOfByte(nByte,nBitNr) // 8765 4321
 LOCAL i,cByte,cOutputByte:="",nOutputByte
 cByte := BinaryString(nByte)
 FOR i := 8 TO 1 STEP -1
  IF i == nBitNr
   cOutputByte := cOutputByte + "1"
  ELSE
   cOutputByte := cOutputByte + substr(cByte,9-i,1)
  ENDIF  
 NEXT i
 nOutputByte := NumericValue(cOutputByte)
RETURN nOutputByte

//
// stringify(xobject)
//
// turning every datatype into a character data type
//
// xobject  ? : argument of an arbitrary datatype
//
// RETURN  C : character representation of the parameter
//

FUNCTION stringify(xobject)
 LOCAL ctype := valtype(xobject),cresult:="NIL"
 DO CASE
  CASE ctype $ "CM" ; cresult := xobject
  CASE ctype == "D" ; cresult := dtoc(xobject)
  CASE ctype == "N" ; cresult := str(xobject)
  CASE ctype == "L" ; cresult := if(xobject,"TRUE","FALSE")
  CASE ctype == "A" ; cresult := atos(xobject)
  CASE ctype $ "OB" ; cresult := ""
 ENDCASE
 RETURN cresult 
 
// 
// zero pad a numeric value to string
//
// str_zero(nnumber,nwidth,ndecimals)
//
// nnumber    N
// nwidth     N : if passed total width to pad to
// ndecimals  N : if passed no of decimals defaults to nnumber
//
// RETURN     C : zero padded number as a character
//

FUNCTION str_zero(nnumber,nwidth,ndecimals)
 LOCAL i,s,cltrs,cresult
 IF ndecimals != NIL
  s := str(nnumber,nwidth,ndecimals)
 ELSEIF nwidth != NIL
  s := str(nnumber,nwidth)
 ELSE
  s := str(nnumber)
 ENDIF
 cltrs := ltrim(s)
 cresult := replicate("0",len(s)-len(cltrs))
 i := at("-",s)
 RETURN if(i>0,"-"+cresult+substr(s,++i),cresult+cltrs)
 
//
// parse take a string and optional delimiter character and returns
//  an array which contains each token of the string
//
// aparse(cstring,cdelimiter)
//
// cstring     C
// cdelimiter  C : optional defaults to ","
//
// RETURN [] character array of tokens
//
// example : creeks = "HALLO,5,2,B/W"
//           aarray := aparse(creeks)
//           aarray := {"HALLO","5","2","B/W"}
//

FUNCTION aparse(cstring,cdelimiter)
 LOCAL list_ := {},i
 DEFAULT cdelimiter TO ","
 DO WHILE len(cstring) > 0
  IF ( i := at(cdelimiter,cstring)) > 0
   aadd(list_,left(cstring,i-1))
   cstring := substr(cstring,i+1)
  ELSE
   aadd(list_,cstring)
   cstring := ""
  ENDIF
 ENDDO
 RETURN list_

//
// udf for indexing in an arbitrary character order(dictionary order)
//
// dict_order(cstring,calpha_order)
//
// cstring      C  : variable to make indexexpression for
// calpha_order C  : determines the order to be used(dictionary order)
//                   defaults to :
//                   "0123456789"+ALPHABET
//                   char. not appearing in this string are ignored
//                   in the key !
//

FUNCTION dict_order(cstring,calpha_order)
 LOCAL ncounter,ctemp,cchar,ckey:=""
 DEFAULT calpha_order TO "0123456789"+ALPHABET
 ctemp := upper(alltrim(cstring))
 FOR ncounter := 1 TO len(ctemp)
  cchar := substr(ctemp,ncounter,1)
  IF cchar $ calpha_order
   ckey += chr(at(cchar,calpha_order) + 31)
  ENDIF
 NEXT ncounter
 RETURN (ckey)
 
//
// encryption system
//
 
Function encrypt(cstring,cencr_key)
 LOCAL cencrypted:="",i,cdummy,nascii,ckar,npos:=1
 DEFAULT cencr_key TO ccryptkey
 cdummy := charmirr(cstring)
 FOR i:=1 TO len(cdummy)
  ckar := substr(cdummy,i,1)
  nascii := asc(ckar) + asc(substr(cencr_key,npos,1))
  IF nascii > 255
   nascii -= 255
  ELSEIF nascii < 0
   nascii += 255
  ENDIF
  cencrypted += chr(nascii)
  npos++
  IF npos > len(cencr_key)
   npos := 1
  ENDIF
 NEXT i
 RETURN cencrypted

FUNCTION decrypt(cstring,cencr_key)
 LOCAL cdecrypted:="",i,nascii,ckar,npos:=1
 DEFAULT cencr_key TO ccryptkey
 FOR i:=1 TO len(cstring)
  ckar := substr(cstring,i,1)
  nascii := asc(ckar) - asc(substr(cencr_key,npos,1))
  IF nascii > 255
   nascii -= 255
  ELSEIF nascii < 0
   nascii += 255
  ENDIF
  cdecrypted += chr(nascii)
  npos++
  IF npos > len(cencr_key)
   npos := 1
  ENDIF
 NEXT i
 RETURN (charmirr(cdecrypted))

FUNCTION encryptkey(cnewkey)
 LOCAL coldkey:=ccryptkey
 IF !empty(cnewkey)
  ccryptkey := cnewkey
 ENDIF
 RETURN coldkey
 
//
//
// give first position of cstring that contains a character not
// contained in callowed otherwise return 0
//

FUNCTION cspnout(cstring,callowed)
 LOCAL npos,lfound:=FALSE
 FOR npos:=1 TO len(cstring)
  IF !(substr(cstring,npos,1) $ callowed)
   lfound := TRUE
   EXIT
  ENDIF
 NEXT npos
 RETURN if(lfound,npos,0)

FUNCTION zeropad(xvalue,nlength,ckar,cposition)
 LOCAL cpadded:=xvalue
 cpadded := stringify(cpadded)
 DEFAULT nlength TO len(cpadded),ckar TO "0"
 DEFAULT cposition TO "L"  // Leftside
 ckar := left(alltrim(ckar),1)
 cposition := upper(left(alltrim(cposition),1))
 cpadded := alltrim(cpadded)
 DO CASE
  CASE !(cposition == "R") // not on the Rightside
   cpadded := padl(cpadded,nlength,ckar)
  OTHERWISE
   cpadded := padr(cpadded,nlength,ckar)
 ENDCASE
RETURN cpadded

//
// CHARACTER [] spl_into_lines(string,line_len,max_lines)
//
// CHARACTER string \\ text to split into max_lines lines of line_len
// NUMERIC line_len \\ defaults to maxcol()-3
// NUMERIC max_lines \\ optional
//
// RETURN char array containing the split lines
//

FUNCTION spl_into_lines(string,line_len,max_lines)
 LOCAL num_lines,line_num,lines
 DEFAULT line_len TO maxcol()-3
 num_lines := if(max_lines == NIL,mlcount(string,line_len), ;
  min(mlcount(string,line_len),max_lines))
 lines := array(num_lines)
 FOR line_num := 1 TO num_lines
  lines[line_num] := memoline(string,line_len,line_num)
 NEXT
RETURN lines

Function isall(xvalue,xchar)
 LOCAL lall:=FALSE
 DEFAULT xchar TO "9"
 IF IS_NUM(xvalue)
  xvalue := str(xvalue)
 ENDIF
 IF IS_NUM(xchar)
  IF IS_ASCII(xchar)
   xchar := chr(xchar)
  ELSE
   xchar := "9"
  ENDIF
 ENDIF
 xvalue := alltrim(xvalue)
 IF xvalue == replicate(xchar,len(xvalue))
  lall := TRUE
 ENDIF
RETURN lall

FUNCTION makenumeric(xvalue,linteger)
 LOCAL cvaltype,nvalue:=0
 DEFAULT linteger TO FALSE
 cvaltype := valtype(xvalue)
 DO CASE
  CASE cvaltype == "N"
   nvalue := xvalue
  CASE cvaltype == "C"
   nvalue := val(xvalue)
  CASE cvaltype == "L"
   IF xvalue
    nvalue := 1
   ELSE
    nvalue := 0
   ENDIF
 ENDCASE
 IF linteger
  nvalue := int(nvalue)
 ENDIF
RETURN nvalue 
 
//
// functions for .ini file system , uses STATIC array asini
//

FUNCTION ini_value(cidentifier,xdef_value)
 LOCAL xreturn,npos
 xreturn := xdef_value
 IF len(asini) > 0
  cidentifier := upper(alltrim(cidentifier))
  npos := ascan(asini,{|el|el[1]==cidentifier})
  IF npos > 0
   xreturn := asini[npos][2]
  ENDIF
 ENDIF
 RETURN xreturn
 
FUNCTION ini_exist(cidentifier)
 LOCAL lexist:=FALSE
 IF len(asini) > 0
  cidentifier := upper(alltrim(cidentifier))
  lexist := (ascan(asini,{|el|el[1]==cidentifier}) > 0)
 ENDIF
 RETURN lexist
 
FUNCTION readinival(cfilename)
 LOCAL cstr,cline,cidentifier,xvalue,npos
 cfilename := upper(alltrim(cfilename))
 cstr := memoread(cfilename)
 DO WHILE !empty(cstr)
  npos := at(EOLINE,cstr)
  IF npos < 1
   cline := alltrim(cstr)
   cstr := ""
  ELSE
   cline := alltrim(substr(cstr,1,npos-1))
   cstr := substr(cstr,npos+len(EOLINE))
  ENDIF
  DO CASE
   CASE empty(cline)
   CASE substr(cline,1,1) == "*"
   CASE substr(cline,1,2) == "/*"
   CASE substr(cline,1,2) == "//"
   OTHERWISE
    cidentifier := upper(alltrim(token(cline,INISEPARATOR,1)))
    IF !empty(cidentifier)
     xvalue := upper(alltrim(token(cline,INISEPARATOR,2)))
     IF !empty(xvalue)
      IF "//" $ xvalue
       npos := at("//",xvalue)
       xvalue := alltrim(substr(xvalue,1,npos-1))
      ENDIF
      DO CASE
       CASE empty(xvalue)
        xvalue := ""
       CASE xvalue == ".F." .OR. xvalue == "FALSE"
        xvalue := FALSE
       CASE xvalue == ".T." .OR. xvalue == "TRUE"
        xvalue := TRUE
       CASE left(xvalue,1) $ (CYFERS + "-+")
        xvalue := val(charrem(" ",xvalue))
      ENDCASE
     ENDIF
     aadd(asini,{cidentifier,xvalue})
    ENDIF
  ENDCASE
 ENDDO
 RETURN NIL
 
FUNCTION clearinival()
 asini := {}
RETURN NIL

FUNCTION writeinival(cfilename)
 LOCAL cstr:="",i,cidentifier,xvalue,lok:=TRUE
 cfilename := upper(alltrim(cfilename))
 FOR i:=1 TO len(asini)
  cidentifier := asini[i][1]
  xvalue := asini[i][2]
  DO CASE
   CASE IS_NIL(xvalue)
    xvalue := ""
   CASE IS_NUM(xvalue)
    xvalue := str(xvalue)
   CASE IS_CHAR(xvalue)
   CASE IS_LOGIC(xvalue)
    IF xvalue
     xvalue := ".T."
    ELSE
     xvalue := ".F."
    ENDIF
   CASE IS_DATE(xvalue)
    xvalue := dtoc(xvalue)
   CASE empty(xvalue)
    xvalue := ""
   OTHERWISE
    xvalue := ""
  ENDCASE
  xvalue := alltrim(xvalue)
  cstr += cidentifier
  IF !empty(xvalue)
   cstr += " " + INISEPARATOR + " " + xvalue
  ENDIF
  cstr += EOLINE
 NEXT i
 IF !empty(cstr)
  IF file(cfilename)
   IF !(ferase(cfilename) == 0)
    lok := FALSE
    ALARMTONE
    alert("Error(writeinival()): " + alltrim(str(ferror())) + " : " + ;
     "Erasing " + cfilename)
   ENDIF
  ENDIF
  IF lok .AND. !memowrit(cfilename,cstr)
   lok := FALSE
   ALARMTONE
   alert("Error(writeinival()): writing " + cfilename)
  ENDIF
 ENDIF
RETURN lok

FUNCTION makeinival(cidentifier,xvalue)
 LOCAL npos:=0
 DEFAULT xvalue TO ""
 IF IS_CHAR(xvalue)
  xvalue := upper(alltrim(xvalue))
 ENDIF
 cidentifier := upper(alltrim(cidentifier))
 IF len(asini) > 0
  npos := ascan(asini,{|el|el[1]==cidentifier})
 ENDIF
 IF npos > 0
  asini[npos][2] := xvalue
 ELSE
  aadd(asini,{cidentifier,xvalue})
 ENDIF
 RETURN NIL


FUNCTION deleteinival(cidentifier)
 LOCAL npos:=0
 IF len(asini) > 0
  cidentifier := upper(alltrim(cidentifier))
  IF (npos := ascan(asini,{|el|el[1]==cidentifier})) > 0
   adel(asini,npos)
   ASHRINK(asini)
  ENDIF
 ENDIF
 RETURN NIL
 
//
// array functions
//

//
// NUMERIC abinsrch(marray,search_for)
// ANYTYPE   marray[]
// ANYTYPE   search_for
//
// search marray[] for value : search_for ,return element number if found
// else 0
//

FUNCTION abinsrch(marray,search_for)
 LOCAL low,high,middle,lexact:=set(_SET_EXACT,TRUE)
 low := 1
 high := len(marray)
 middle := int((low + high) / 2)
 DO WHILE (!(marray[middle] == search_for)) .AND. (!(low == high))
  IF search_for > marray[middle]
   low := middle + 1
  ELSE
   high := middle - 1
  ENDIF
  middle := int((low + high) / 2)
 ENDDO
 IF !(marray[middle] == search_for)
  middle := 0
 ENDIF
 set(_SET_EXACT,lexact)
 RETURN middle
 
//
// recursive replacement for afill()
//

FUNCTION ainitfill(aarray_,xvalue)
 LOCAL i
 FOR i:=1 TO len(aarray_)
  IF IS_ARRAY(aarray_[i])
   ainitfill(aarray_[i],xvalue)
  ELSE
   aarray_[i] := xvalue
  ENDIF
 NEXT i
 RETURN NIL
 
//
//  asave_array(aarray_,cfilename)
//  Writes an array to a disk file.
//  General-purpose function for saving an array to a disk file.
//  This is only the public interface. The real work is done by
//  recursive calls to the ElementOut() function.
//  Pass an array (or a reference to an array) and the name of the
//  file to store the array.
//
//  asave_array({1,2,3,4}, "NUMS.ARY")
//
//  This function returns .t. if successful, .f. if not.
//

FUNCTION asave_array(aarray_,cfilename)
 LOCAL cnt := len(aarray_)
 LOCAL success := .f.
 LOCAL handle := fcreate(cfilename,FC_NORMAL)
 IF handle != -1
  success := elementout(handle,aarray_)
  fclose(handle)
 ENDIF
 RETURN success
 
//
//   Given a file handle and an array, write the contents of the
//   array to the file in the following form.
//
//      LL T WW E... T WW E...
//
//   Where LL is a two byte integer representing the number of
//   elements in the array, T is a character representing the data
//   type of the first element, WW is a two byte integer
//   representing the width of the element; followed by repetitions
//   of that basic pattern.
//
//   Nested arrays are handled by calling ElementOut() when ever an
//   array is encountered within the elements in the array
//   currently being processed. (Know as recursion.)  The LL length
//   bytes are written following an "A" data type and the process
//   gets repeated. Isn't recursion wonderful?

//   This is a static function and therefore not visible to any
//   functions outside of the source code file containing it. All
//   calls from the outside must be made to asave_array().
//

 Static FUNCTION elementout(handle,aarray_)
 LOCAL success := .t.
 LOCAL i,buffer
 LOCAL cnt := len(aarray_)
//
// Write the overall array size.
//
 fwrite(handle,i2bin(cnt))
//
//  Process each element in the array.
//
 FOR i := 1 TO cnt
//
//     Special handling for the nil and code block data types.
//     Both will be labeled type "Z" and for consistency wit the
//     other data types, will have a width of one and an element
//     value of "Z". However, a nil will be placed in the array
//     when it comes time to load it from the file.
//
  IF (aarray_[i] == NIL) .OR. IS_BLOCK(aarray_[i])
   buffer := "Z" +i2bin(1) +"Z"
  ELSE
//
//   Each element is encoded as follows.
//   Data type:  C,D,L,N.
//   Width:  Number of characters needed.
//   Element Value:  String version of the value.
// 
   buffer := valtype(aarray_[i])
   DO CASE
    CASE IS_CHAR(aarray_[i])
     buffer += i2bin(len(aarray_[i]))    //  Width of the string
     buffer += aarray_[i]
    CASE IS_DATE(aarray_[i])
     buffer += i2bin(8)             //  Dates are 8 wide
     buffer += dtoc(aarray_[i])
    CASE IS_LOGIC(aarray_[i])
     buffer += i2bin(1)             //  Logicals are 1 wide
     buffer += if(aarray_[i], "T", "F")
    CASE IS_NUM(aarray_[i])
//
//  Convert number to string, trim spaces, and
//  calculated width of number based on the string.
//
     buffer += i2bin(len(ltrim(str(aarray_[i]))))
     buffer += ltrim(str(aarray_[i]))
    OTHERWISE
//
//  Type "A" for arrays will be handled
//  after we write the type.
//
   ENDCASE
  ENDIF
//
//  Write the buffer, constructed above, to the file.
//
  IF fwrite(handle,buffer,len(buffer)) != len(buffer)
   success := .f.
   EXIT
  ENDIF
//
//   If this is a nested array, it's recursion time!
//   Call ElementOut() again, it will append a series
//   of types/widths/values to the current file.
//
  IF left(buffer,1) == "A"
   elementout(handle,aarray_[i])
  ENDIF
 NEXT i
RETURN success
//
//   arest_array()
//   Loads an array from a disk file saved with asave_array()
//   General-purpose function for restoring an array from a disk
//   file. This is only the public interface. The real work is done
//   by recursive calls to the ElementIn() function.
//   Given the name of a file created by the asave_array() function,
//   read the file construct an array to return.
//
//     n_ := arest_array("NUMS.ARY")
//
//   This function returns a reference to the array it creates.
//
FUNCTION arest_array(cfilename)
 LOCAL handle,aarray_ := {}
 IF (handle := fopen(cfilename,FO_READ)) != -1
  elementin(handle,aarray_)
 ENDIF
 fclose(handle)
 RETURN aarray_
 
//
//  Given a file handle and a reference to an array, read elements
//  from the file and add them to the array.
//  The file must be created by the asave_array() function and be
//  structured as described in the asave_array() comments.
//  Nested arrays are detected and sub-arrays stored in the main
//  array as needed. The file reading for a sub-array is accomplished
//  through another call to elementin(). This is known as recursion.
//  This is a static function and therefore not visible to any
//  functions outside of the source code file containing it. All
//  calls from the outside must be made to arest_array().
//

STATIC FUNCTION elementin(handle,aarray_)
 LOCAL buffer,i,cnt,ilen,itype
//
// Read the overall array size
//
 buffer := space(2)
 IF fread(handle,@buffer,2) == 2
//
//  Process each array element stored in the file.
//
  cnt := bin2w(buffer)
  FOR i := 1 TO cnt
//
//  Read the element's data type.
//  If element is a nested array-- recursion time!
//
   IF (itype := freadstr(handle,1)) == "A"
    aadd(aarray_,{})
    elementin(handle,aarray_[len(aarray_)])
   ELSE
//
//  Read the length of the element.
//
    buffer := space(2)
    IF fread(handle,@buffer,2) == 2
     ilen := bin2w(buffer)
//
// Read the actual element.
//
     buffer := space(ilen)
     IF fread(handle,@buffer,ilen) == ilen
//
//  Convert from string to specified data type.
//
      DO CASE
//
//   Note that we can't save code blocks. If you
//   attempted to save one from an array, we will have
//   empty space and thus must add a NIL to serve
//   as a placeholder.
//
       CASE (itype == "B") .OR. (itype == "Z")
        aadd(aarray_,NIL)
       CASE itype == "C"
        aadd(aarray_, buffer)
       CASE itype == "D"
        aadd(aarray_, ctod(buffer))
       CASE itype == "L"
        aadd(aarray_, (buffer == "T"))
       CASE itype == "N"
        aadd(aarray_, val(buffer))
      ENDCASE
     ENDIF
    ENDIF
   EndIf
  Next i
 ENDIF
 RETURN NIL
 
// 
// ANYTYPE afind(array_,bblock)
//
// array_  - array to search for an array element
// bblock - code block specifying the comparision condition
//
// RETURN  - an array-element of array_
//
// general purpose array scanning function
// examples : -> AMAX,AMIN,AMAXLEN,AMINLEN in noot.ch
//

FUNCTION afind(array_,bblock)
 LOCAL x,lexact:=set(_SET_EXACT,TRUE)
 IF IS_ARRAY(array_)
  IF !empty(array_)
   x := array_[1]
   aeval(array_,{|el|x := if(eval(bblock,el,x),el,x)})
  ENDIF
 ENDIF
 set(_SET_EXACT,lexact)
RETURN x

//
// asum()      : returns sum of the elements of a 1dim-numeric array
// aarray    A : array to sum
// nstart    N : starting position,default 1
// nstop     N : stopping position,default bottom element
// return    N : sum
//

FUNCTION asum(aarray,nstart,nstop)
 LOCAL nresult := 0
 aeval(aarray,{|x| nresult += x},nstart,nstop)
 RETURN nresult
 
// 
// move array element from one location to another without removing an
// element
// amove()
//
// aarray    []
// nfrom     N   : array elementnumber to move if NIL then ptarget
//                 is searched for
// ndest     N   : array elementnumber to move to
// ptarget   ?   : if nfrom == NIL this element is searched for and moved
//
// RETURN    L   : .T. if succesfull
//

FUNCTION amove(aarray,nfrom,ndest,ptarget)
 LOCAL npos,lresult:=.F.,psave
 IF IS_NUM(nfrom)
  npos := if(nfrom<=len(aarray),nfrom,0)
 ELSE
  npos := ascan(aarray,ptarget)
 ENDIF
 IF npos > 0
  ndest := MINIMAX(ndest,1,len(aarray))
  psave := aarray[npos]
  adel(aarray,npos)
  ains(aarray,ndest)
  aarray[ndest] := psave
  lresult := .T.
 ENDIF
 Return lresult
 
// 
// return the contents of an array as one string
// can handle subarrays by recursion
//
// atos() : array to string
//
// avalues   []
// apictures []  : optional array of pictures
// cseparator C  : optional delimiter , default ""
//
// RETURN    C   : string
//

Function atos(avalues,apictures,cseparator)
 LOCAL i,nlen:=len(avalues),cresult:="",xpicture
 DEFAULT cseparator TO ""
 IF !IS_ARRAY(apictures)
  xpicture := if(IS_CHAR(apictures),apictures,"")
  apictures := array(nlen)
  afill(apictures,xpicture)
 ENDIF
 FOR i:=1 TO nlen
  IF IS_ARRAY(avalues[i])
   cresult += atos(avalues[i],apictures[i],cseparator)
  ELSE
   cresult += transform(avalues[i],apictures[i])
  ENDIF
  IF i < nlen
   cresult += cseparator
  ENDIF
 NEXT i
 RETURN cresult
 
// 
// give a column of an array
// acolumn()
//
// aarray    []
// ncolumn    N
// nstart     N defaults to 1
// nstop      N defaults to bottom row
//
//

FUNCTION acolumn(aarray,ncolumn,nstart,nstop)
 LOCAL col_ := {}
 AEval(aarray,{|e| aadd(col_,e[ncolumn])},nstart,nstop)
 RETURN col_
 
//
// get system
// 
 
//
// GetListBox() for use in GET VALID clause
//
// usage :   cstring := space(12)
//           ckeuzes_ := ...
//           ..GET cstring VALID { |oGet| GetListBox(oGet,ckeuzes_) }
//
  
Function GetListBox(oGet,clist_)
 LOCAL lvalid,cvar,nt,nl,nb,nr,clistboxarea,nIndex
 cvar := Upper(AllTrim(oGet:varget()))
 AEval(clist_,{|element|element:=Upper(AllTrim(element))},,,TRUE)
 ASort(clist_)
 lvalid := ( AScan(clist_,cvar) > 0)
 If !lvalid
  nt := oGet:Row + 1
  nl := oGet:Col + 1
  If ( MaxRow() - nt) > 5
   nb := nt + 5
  Else 
   nb := MaxRow() - 2
  EndIf
  nr := nl + Len(AMAXLEN(clist_))
  clistboxarea := win_save(nt-1,nl-1,nb+1,nr+1)
  @ nt-1,nl-1 Clear to nb+1,nr+1 
  @ nt-1,nl-1 to nb+1,nr+1 DOUBLE
  nIndex := AChoice(nt,nl,nb,nr,clist_)
  If nIndex > 0
   cvar := clist_[nIndex]
   oGet:varPut(cvar)
   lvalid := TRUE
  EndIf
  win_restore(clistboxarea)
 EndIf
 Return lvalid

//
// Get reader for hidden input of a string
//
// usage   cPassword := space(15)
//         ...GET cPassword SEND reader := {|oGet| HiddenReader(oGet)}
// 
PROCEDURE HiddenReader(oGet)
 LOCAL cBuffer, cChar, nKey
 If GetPreValidate( oGet )
  oGet:setFocus()
  cBuffer := oGet:buffer
  cChar   := Replicate( "*", Len(Trim(cBuffer)) )
  oGet:varPut( PadR(cChar, Len(cBuffer)) )
  oGet:updateBuffer()
  DO WHILE oGet:exitState == GE_NOEXIT
   IF oGet:typeOut
    oGet:exitState := GE_ENTER
   ENDIF
   DO WHILE oGet:exitState == GE_NOEXIT
    nKey := Inkey(0)
    DO CASE
     CASE nKey == K_DEL .OR. nKey == K_BS
      GetApplykey( oGet, nKey )
      cBuffer := Stuff( cBuffer, oGet:pos, 1, "" )
     CASE nKey >= 32 .AND. nKey <= 255
      GetApplykey( oGet, Asc( "*" ) )
      cChar:= Chr(nKey)
      IF oGet:typeOut
      ELSEIF Set( _SET_INSERT )
       cBuffer := Stuff( cBuffer, oGet:pos-1, 0, cChar )
      ELSE
       cBuffer := Stuff( cBuffer, oGet:pos-1, 1, cChar )
      ENDIF
     OTHERWISE
      GetApplykey( oGet, nKey )
    ENDCASE
   ENDDO
   IF ! GetPostValidate( oGet )
    oGet:exitState:= GE_NOEXIT
   ENDIF
  EndDo
  oGet:killFocus()
  oGet:varPut(cBuffer)
 EndIf
 Return
 
//
// timer functions, uses STATIC nStartTime
//

Procedure StartStopWatch()
 nStartTime := seconds()
 RETURN

FUNCTION ElapsedTime() // resolution = 1/100s
 LOCAL nElapsed
 IF nStartTime > seconds()
  nElapsed := seconds() + (86400 - nStartTime)
 ELSE
  nElapsed := seconds() - nStartTime
 ENDIF
 RETURN nElapsed

//
// einde noot.prg
//
